@@ -8,8 +8,7 @@
 
 std::function<Eigen::MatrixXd(const Eigen::Ref<const Eigen::MatrixXd> &,
                               const Eigen::Ref<const Eigen::VectorXd> &)>
-set_g_fn(const std::string method)
-{
+set_g_fn(const std::string method) {
   std::map<std::string, std::function<Eigen::MatrixXd(
                             const Eigen::Ref<const Eigen::MatrixXd> &,
                             const Eigen::Ref<const Eigen::VectorXd> &)>>
@@ -26,73 +25,57 @@ set_g_fn(const std::string method)
              {"poisson_identity", g_poi_identity},
              {"poisson_sqrt", g_poi_sqrt},
              {"quasipoisson_log", g_qpoi_log},
-             {"quasipoisson_identity", g_qpoi_identity}}};
+             {"quasipoisson_identity", g_qpoi_identity},
+             {"quasipoisson_sqrt", g_qpoi_sqrt}}};
   return g_map[method];
 }
 
 Eigen::VectorXd proj(const Eigen::Ref<const Eigen::MatrixXd> &l,
-                     const Eigen::Ref<const Eigen::VectorXd> &x)
-{
+                     const Eigen::Ref<const Eigen::VectorXd> &x) {
   return x - l.transpose() * ((l * l.transpose()).householderQr().solve(l * x));
 }
 
-Eigen::ArrayXd log_linkinv(const Eigen::Ref<const Eigen::VectorXd> &x)
-{
+Eigen::ArrayXd log_linkinv(const Eigen::Ref<const Eigen::VectorXd> &x) {
   return exp(x.array());
 }
 
-Eigen::ArrayXd logit_linkinv(const Eigen::Ref<const Eigen::VectorXd> &x)
-{
+Eigen::ArrayXd logit_linkinv(const Eigen::Ref<const Eigen::VectorXd> &x) {
   return inverse(1.0 + exp(-x.array()));
 }
 
-Eigen::ArrayXd probit_linkinv(const Eigen::Ref<const Eigen::VectorXd> &x)
-{
+Eigen::ArrayXd probit_linkinv(const Eigen::Ref<const Eigen::VectorXd> &x) {
   Eigen::ArrayXd out(x.size());
-  for (int i = 0; i < x.size(); ++i)
-  {
+  for (int i = 0; i < x.size(); ++i) {
     out[i] = 0.5 * std::erfc(-x[i] * M_SQRT1_2);
   }
   return out;
 }
 
 Eigen::VectorXd mele_mean(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                          const Eigen::Ref<const Eigen::ArrayXd> &w)
-{
-  if (w.size() == 0)
-  {
+                          const Eigen::Ref<const Eigen::ArrayXd> &w) {
+  if (w.size() == 0) {
     return x.colwise().mean();
-  }
-  else
-  {
+  } else {
     return (w.matrix().transpose() * x) / x.rows();
   }
 }
 
 Eigen::VectorXd mele_sd(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                        const Eigen::Ref<const Eigen::ArrayXd> &w)
-{
-  if (w.size() == 0)
-  {
+                        const Eigen::Ref<const Eigen::ArrayXd> &w) {
+  if (w.size() == 0) {
     return (x.colwise().mean()).array().sqrt();
-  }
-  else
-  {
+  } else {
     return ((w.matrix().transpose() * x) / x.rows()).array().sqrt();
   }
 }
 
 Eigen::VectorXd mele_lm(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                        const Eigen::Ref<const Eigen::ArrayXd> &w)
-{
+                        const Eigen::Ref<const Eigen::ArrayXd> &w) {
   const Eigen::VectorXd y = x.col(1) - x.col(0);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
-  if (w.size() == 0)
-  {
+  if (w.size() == 0) {
     return xmat.colPivHouseholderQr().solve(y);
-  }
-  else
-  {
+  } else {
     const Eigen::MatrixXd wsqrt =
         Eigen::MatrixXd(w.sqrt().matrix().asDiagonal());
     return (wsqrt * xmat).colPivHouseholderQr().solve(wsqrt * y);
@@ -100,8 +83,7 @@ Eigen::VectorXd mele_lm(const Eigen::Ref<const Eigen::MatrixXd> &x,
 }
 
 Eigen::MatrixXd g_mean(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                       const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                       const Eigen::Ref<const Eigen::VectorXd> &par) {
   return x.rowwise() - par.transpose();
 }
 
@@ -110,23 +92,18 @@ Eigen::VectorXd gr_nloglr_mean(const Eigen::Ref<const Eigen::VectorXd> &l,
                                const Eigen::Ref<const Eigen::MatrixXd> &x,
                                const Eigen::Ref<const Eigen::VectorXd> &par,
                                const Eigen::Ref<const Eigen::ArrayXd> &w,
-                               const bool weighted)
-{
+                               const bool weighted) {
   Eigen::ArrayXd c(x.rows());
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
   }
   return -c.sum() * l;
 }
 
 Eigen::MatrixXd g_sd(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                     const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                     const Eigen::Ref<const Eigen::VectorXd> &par) {
   return x.rowwise() - par.array().square().matrix().transpose();
 }
 
@@ -135,23 +112,18 @@ Eigen::VectorXd gr_nloglr_sd(const Eigen::Ref<const Eigen::VectorXd> &l,
                              const Eigen::Ref<const Eigen::MatrixXd> &x,
                              const Eigen::Ref<const Eigen::VectorXd> &par,
                              const Eigen::Ref<const Eigen::ArrayXd> &w,
-                             const bool weighted)
-{
+                             const bool weighted) {
   Eigen::ArrayXd c(x.rows());
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
   }
   return -2 * c.sum() * (par.array() * l.array());
 }
 
 Eigen::MatrixXd g_lm(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                     const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                     const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -163,24 +135,19 @@ Eigen::VectorXd gr_nloglr_lm(const Eigen::Ref<const Eigen::VectorXd> &l,
                              const Eigen::Ref<const Eigen::MatrixXd> &x,
                              const Eigen::Ref<const Eigen::VectorXd> &par,
                              const Eigen::Ref<const Eigen::ArrayXd> &w,
-                             const bool weighted)
-{
+                             const bool weighted) {
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c(x.rows());
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
   }
   return -(xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_gauss_log(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                            const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                            const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -188,33 +155,28 @@ Eigen::MatrixXd g_gauss_log(const Eigen::Ref<const Eigen::MatrixXd> &x,
          ((y - log_linkinv(xmat * par + s)) * log_linkinv(xmat * par + s));
 }
 
-Eigen::VectorXd gr_nloglr_gauss_log(
-    const Eigen::Ref<const Eigen::VectorXd> &l,
-    const Eigen::Ref<const Eigen::MatrixXd> &g,
-    const Eigen::Ref<const Eigen::MatrixXd> &x,
-    const Eigen::Ref<const Eigen::VectorXd> &par,
-    const Eigen::Ref<const Eigen::ArrayXd> &w,
-    const bool weighted)
-{
+Eigen::VectorXd
+gr_nloglr_gauss_log(const Eigen::Ref<const Eigen::VectorXd> &l,
+                    const Eigen::Ref<const Eigen::MatrixXd> &g,
+                    const Eigen::Ref<const Eigen::MatrixXd> &x,
+                    const Eigen::Ref<const Eigen::VectorXd> &par,
+                    const Eigen::Ref<const Eigen::ArrayXd> &w,
+                    const bool weighted) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c = y * log_linkinv(xmat * par + s) -
                      2.0 * log_linkinv(2.0 * (xmat * par + s));
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_gauss_inverse(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                                const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                                const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -223,66 +185,56 @@ Eigen::MatrixXd g_gauss_inverse(const Eigen::Ref<const Eigen::MatrixXd> &x,
           (DBL_EPSILON + (xmat * par + s).array()).pow(-2));
 }
 
-Eigen::VectorXd gr_nloglr_gauss_inverse(
-    const Eigen::Ref<const Eigen::VectorXd> &l,
-    const Eigen::Ref<const Eigen::MatrixXd> &g,
-    const Eigen::Ref<const Eigen::MatrixXd> &x,
-    const Eigen::Ref<const Eigen::VectorXd> &par,
-    const Eigen::Ref<const Eigen::ArrayXd> &w,
-    const bool weighted)
-{
+Eigen::VectorXd
+gr_nloglr_gauss_inverse(const Eigen::Ref<const Eigen::VectorXd> &l,
+                        const Eigen::Ref<const Eigen::MatrixXd> &g,
+                        const Eigen::Ref<const Eigen::MatrixXd> &x,
+                        const Eigen::Ref<const Eigen::VectorXd> &par,
+                        const Eigen::Ref<const Eigen::ArrayXd> &w,
+                        const bool weighted) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c =
       (DBL_EPSILON + (xmat * par + s).array()).pow(-3) *
       (2.0 * y - 3.0 * inverse(DBL_EPSILON + (xmat * par + s).array()));
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_bin_logit(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                            const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                            const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   return xmat.array().colwise() * (y - logit_linkinv(xmat * par + s));
 }
 
-Eigen::VectorXd gr_nloglr_bin_logit(
-    const Eigen::Ref<const Eigen::VectorXd> &l,
-    const Eigen::Ref<const Eigen::MatrixXd> &g,
-    const Eigen::Ref<const Eigen::MatrixXd> &x,
-    const Eigen::Ref<const Eigen::VectorXd> &par,
-    const Eigen::Ref<const Eigen::ArrayXd> &w,
-    const bool weighted)
-{
+Eigen::VectorXd
+gr_nloglr_bin_logit(const Eigen::Ref<const Eigen::VectorXd> &l,
+                    const Eigen::Ref<const Eigen::MatrixXd> &g,
+                    const Eigen::Ref<const Eigen::MatrixXd> &x,
+                    const Eigen::Ref<const Eigen::VectorXd> &par,
+                    const Eigen::Ref<const Eigen::ArrayXd> &w,
+                    const bool weighted) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c =
       -logit_linkinv(xmat * par + s) * (1.0 - logit_linkinv(xmat * par + s));
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_bin_probit(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                             const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                             const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -295,14 +247,13 @@ Eigen::MatrixXd g_bin_probit(const Eigen::Ref<const Eigen::MatrixXd> &x,
          (c * (y - probit_linkinv(xmat * par + s)) * phi);
 }
 
-Eigen::VectorXd gr_nloglr_bin_probit(
-    const Eigen::Ref<const Eigen::VectorXd> &l,
-    const Eigen::Ref<const Eigen::MatrixXd> &g,
-    const Eigen::Ref<const Eigen::MatrixXd> &x,
-    const Eigen::Ref<const Eigen::VectorXd> &par,
-    const Eigen::Ref<const Eigen::ArrayXd> &w,
-    const bool weighted)
-{
+Eigen::VectorXd
+gr_nloglr_bin_probit(const Eigen::Ref<const Eigen::VectorXd> &l,
+                     const Eigen::Ref<const Eigen::MatrixXd> &g,
+                     const Eigen::Ref<const Eigen::MatrixXd> &x,
+                     const Eigen::Ref<const Eigen::VectorXd> &par,
+                     const Eigen::Ref<const Eigen::ArrayXd> &w,
+                     const bool weighted) {
   const Eigen::VectorXd o = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -319,20 +270,16 @@ Eigen::VectorXd gr_nloglr_bin_probit(
       -phi.square() + (y - probit_linkinv(xmat * par + o)) * dphi;
   const Eigen::ArrayXd dt = phi * (1.0 - 2.0 * probit_linkinv(xmat * par + o));
   Eigen::ArrayXd c = t.pow(-2) * (ds * t - s * dt);
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_bin_log(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                          const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                          const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -346,27 +293,22 @@ Eigen::VectorXd gr_nloglr_bin_log(const Eigen::Ref<const Eigen::VectorXd> &l,
                                   const Eigen::Ref<const Eigen::MatrixXd> &x,
                                   const Eigen::Ref<const Eigen::VectorXd> &par,
                                   const Eigen::Ref<const Eigen::ArrayXd> &w,
-                                  const bool weighted)
-{
+                                  const bool weighted) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c = (DBL_EPSILON + 1.0 - log_linkinv(xmat * par + s)).pow(-2) *
                      log_linkinv(xmat * par + s) * (y - 1.0);
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_poi_log(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                          const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                          const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -378,25 +320,20 @@ Eigen::VectorXd gr_nloglr_poi_log(const Eigen::Ref<const Eigen::VectorXd> &l,
                                   const Eigen::Ref<const Eigen::MatrixXd> &x,
                                   const Eigen::Ref<const Eigen::VectorXd> &par,
                                   const Eigen::Ref<const Eigen::ArrayXd> &w,
-                                  const bool weighted)
-{
+                                  const bool weighted) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c = -log_linkinv(xmat * par + s);
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_poi_identity(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                               const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                               const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -404,32 +341,27 @@ Eigen::MatrixXd g_poi_identity(const Eigen::Ref<const Eigen::MatrixXd> &x,
          (y * inverse(DBL_EPSILON + (xmat * par + s).array()) - 1.0);
 }
 
-Eigen::VectorXd gr_nloglr_poi_identity(
-    const Eigen::Ref<const Eigen::VectorXd> &l,
-    const Eigen::Ref<const Eigen::MatrixXd> &g,
-    const Eigen::Ref<const Eigen::MatrixXd> &x,
-    const Eigen::Ref<const Eigen::VectorXd> &par,
-    const Eigen::Ref<const Eigen::ArrayXd> &w,
-    const bool weighted)
-{
+Eigen::VectorXd
+gr_nloglr_poi_identity(const Eigen::Ref<const Eigen::VectorXd> &l,
+                       const Eigen::Ref<const Eigen::MatrixXd> &g,
+                       const Eigen::Ref<const Eigen::MatrixXd> &x,
+                       const Eigen::Ref<const Eigen::VectorXd> &par,
+                       const Eigen::Ref<const Eigen::ArrayXd> &w,
+                       const bool weighted) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c = -y * (DBL_EPSILON + (xmat * par + s).array()).pow(-2);
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_poi_sqrt(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                           const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                           const Eigen::Ref<const Eigen::VectorXd> &par) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
@@ -443,27 +375,22 @@ Eigen::VectorXd gr_nloglr_poi_sqrt(const Eigen::Ref<const Eigen::VectorXd> &l,
                                    const Eigen::Ref<const Eigen::MatrixXd> &x,
                                    const Eigen::Ref<const Eigen::VectorXd> &par,
                                    const Eigen::Ref<const Eigen::ArrayXd> &w,
-                                   const bool weighted)
-{
+                                   const bool weighted) {
   const Eigen::VectorXd s = x.col(0);
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c =
       -2.0 * y * (DBL_EPSILON + (xmat * par + s).array()).pow(-2) - 2.0;
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array()) * c;
   }
   return (xmat.transpose() * (xmat.array().colwise() * c).matrix()) * l;
 }
 
 Eigen::MatrixXd g_qpoi_log(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                           const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                           const Eigen::Ref<const Eigen::VectorXd> &par) {
   const int p = x.cols() - 2;
   const Eigen::VectorXd beta = par.head(p);
   const double phi = par(p);
@@ -484,8 +411,7 @@ Eigen::VectorXd gr_nloglr_qpoi_log(const Eigen::Ref<const Eigen::VectorXd> &l,
                                    const Eigen::Ref<const Eigen::MatrixXd> &x,
                                    const Eigen::Ref<const Eigen::VectorXd> &par,
                                    const Eigen::Ref<const Eigen::ArrayXd> &w,
-                                   const bool weighted)
-{
+                                   const bool weighted) {
   const int p = x.cols() - 2;
   const Eigen::VectorXd beta = par.head(p);
   const double phi = par(p);
@@ -493,12 +419,9 @@ Eigen::VectorXd gr_nloglr_qpoi_log(const Eigen::Ref<const Eigen::VectorXd> &l,
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c(x.rows());
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
   }
   const Eigen::ArrayXd c2 =
@@ -526,8 +449,7 @@ Eigen::VectorXd gr_nloglr_qpoi_log(const Eigen::Ref<const Eigen::VectorXd> &l,
 }
 
 Eigen::MatrixXd g_qpoi_identity(const Eigen::Ref<const Eigen::MatrixXd> &x,
-                                const Eigen::Ref<const Eigen::VectorXd> &par)
-{
+                                const Eigen::Ref<const Eigen::VectorXd> &par) {
   const int p = x.cols() - 2;
   const Eigen::VectorXd beta = par.head(p);
   const double phi = par(p);
@@ -546,14 +468,13 @@ Eigen::MatrixXd g_qpoi_identity(const Eigen::Ref<const Eigen::MatrixXd> &x,
   return out;
 }
 
-Eigen::VectorXd gr_nloglr_qpoi_identity(
-    const Eigen::Ref<const Eigen::VectorXd> &l,
-    const Eigen::Ref<const Eigen::MatrixXd> &g,
-    const Eigen::Ref<const Eigen::MatrixXd> &x,
-    const Eigen::Ref<const Eigen::VectorXd> &par,
-    const Eigen::Ref<const Eigen::ArrayXd> &w,
-    const bool weighted)
-{
+Eigen::VectorXd
+gr_nloglr_qpoi_identity(const Eigen::Ref<const Eigen::VectorXd> &l,
+                        const Eigen::Ref<const Eigen::MatrixXd> &g,
+                        const Eigen::Ref<const Eigen::MatrixXd> &x,
+                        const Eigen::Ref<const Eigen::VectorXd> &par,
+                        const Eigen::Ref<const Eigen::ArrayXd> &w,
+                        const bool weighted) {
   const int p = x.cols() - 2;
   const Eigen::VectorXd beta = par.head(p);
   const double phi = par(p);
@@ -561,12 +482,9 @@ Eigen::VectorXd gr_nloglr_qpoi_identity(
   const Eigen::ArrayXd y = x.col(1);
   const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
   Eigen::ArrayXd c(x.rows());
-  if (weighted)
-  {
+  if (weighted) {
     c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
-  }
-  else
-  {
+  } else {
     c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
   }
   Eigen::MatrixXd out(p + 1, p + 1);
@@ -596,3 +514,79 @@ Eigen::VectorXd gr_nloglr_qpoi_identity(
                   .sum();
   return out * l;
 }
+
+Eigen::MatrixXd g_qpoi_sqrt(const Eigen::Ref<const Eigen::MatrixXd> &x,
+                            const Eigen::Ref<const Eigen::VectorXd> &par) {
+  // The first two columns of x are the offset and y
+  const int p = x.cols() - 2;
+  // The last element of par is the dispersion
+  const Eigen::VectorXd beta = par.head(p);
+  const double phi = par(p);
+  const Eigen::VectorXd s = x.col(0);
+  const Eigen::ArrayXd y = x.col(1);
+  const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
+  Eigen::MatrixXd out(x.rows(), p + 1);
+  out.leftCols(p) =
+      xmat.array().colwise() *
+      ((2.0 / phi) * (y * inverse(DBL_EPSILON + (xmat * par + s).array()) -
+                      (xmat * par + s).array()));
+  out.col(p) = std::pow(phi, -2) *
+                   ((DBL_EPSILON + (xmat * par + s).array()).pow(-2)) *
+                   square(y - (xmat * beta + s).array().pow(2)) -
+               1.0 / phi;
+  return out;
+}
+
+Eigen::VectorXd
+gr_nloglr_qpoi_sqrt(const Eigen::Ref<const Eigen::VectorXd> &l,
+                    const Eigen::Ref<const Eigen::MatrixXd> &g,
+                    const Eigen::Ref<const Eigen::MatrixXd> &x,
+                    const Eigen::Ref<const Eigen::VectorXd> &par,
+                    const Eigen::Ref<const Eigen::ArrayXd> &w,
+                    const bool weighted) {
+  // The first two columns of x are the offset and y
+  const int p = x.cols() - 2;
+  // The last element of par is the dispersion
+  const Eigen::VectorXd beta = par.head(p);
+  const double phi = par(p);
+  const Eigen::VectorXd s = x.col(0);
+  const Eigen::ArrayXd y = x.col(1);
+  const Eigen::MatrixXd xmat = x.rightCols(x.cols() - 2);
+  Eigen::ArrayXd c(x.rows());
+  if (weighted) {
+    c = w * inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
+  } else {
+    c = inverse((Eigen::VectorXd::Ones(g.rows()) + g * l).array());
+  }
+  Eigen::MatrixXd out(p + 1, p + 1);
+  out.topLeftCorner(p, p) =
+      -2.0 * std::pow(phi, -1) *
+      (xmat.transpose() *
+       (xmat.array().colwise() *
+        (c * (y * (DBL_EPSILON + (xmat * par + s).array()).pow(-2) + 1.0)))
+           .matrix());
+  out.topRightCorner(p, 1) =
+      -2.0 * std::pow(phi, -2) *
+      (xmat.array().colwise() *
+       (c * inverse(DBL_EPSILON + (xmat * par + s).array()) *
+        (y - square(y - (xmat * beta + s).array()))))
+          .colwise()
+          .sum()
+          .transpose();
+  out.bottomLeftCorner(1, p) =
+      -2.0 * std::pow(phi, -2) *
+      (xmat.array().colwise() *
+       (c * inverse(DBL_EPSILON + (xmat * par + s).array()) *
+        (y - square(y - (xmat * beta + s).array())) *
+        ((DBL_EPSILON + (xmat * par + s).array()).pow(-2) *
+             (y - square(y - (xmat * beta + s).array())) +
+         2.0)))
+          .colwise()
+          .sum();
+  out(p, p) = (c * ((-2.0 * std::pow(phi, -3) *
+                     (DBL_EPSILON + (xmat * par + s).array()).pow(-2) *
+                     square(y - square(y - (xmat * beta + s).array()))) +
+                    std::pow(phi, -2)))
+                  .sum();
+  return out * l;
+}
