@@ -14,6 +14,6 @@ test_that("Invalid `control`.", {
   expect_error(el_control(verbose = "TRUE"))
   expect_error(el_control(keep_data = c(TRUE, TRUE)))
   expect_error(el_control(nthreads = c(10, 20)))
-  expect_warning(el_control(nthreads = 0))
+  expect_error(el_control(nthreads = 0))
   expect_warning(el_control(nthreads = .Machine$integer.max))
 })
